/*******************************************************************************
  Generated Main File 

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using MPLAB Code Configurator

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI. 
    Generation Information : 
        Product Revision  :  MPLAB Code Configurator - v1.0
        Device            :  PIC16F1825
        Version           :  1.1
    The generated drivers are tested against the following:
        Compiler          :  XC8 v1.21
        MPLAB             :  MPLAB X 1.90
*******************************************************************************/

/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/


#include "MCC_Generated_Files/mcc.h"
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

extern int seconds;
int flagMovedTime;

void LowerFlag(int currentTime);
void RaiseFlag(int currentTime);

/****************************************************************************
 *
 *                        Main application
 *
****************************************************************************/
void main(void)
{
    // initialize the device
    SYSTEM_Initialize();

    puts("Receiver running\r\n");

    /**
     *                          CORE APPLICATION
     */

    static char inputBuf[4];
    static int pos;
    bool flagUp = false, doorOpen = false, mailInBox = false, lowBatt = false;
    int lowLight, highLight;
    int currentTime;
    int lastFlagWaveTime, lowBattTime;
    char *endPtr;
    enum {UNKNOWN, DOWN, UP, WAVING} flagState = UNKNOWN, newFlagState;

    while (1)
    {
        char c;

        INTERRUPT_GlobalInterruptDisable();
        currentTime = seconds;
        INTERRUPT_GlobalInterruptEnable();

        // If there was a receive buffer overflow, clear it
        if (RCSTAbits.OERR)
        {
            RCSTAbits.CREN = 0;
            char c = RCREG;
            c = RCREG;
            c = RCREG;
            RCSTAbits.CREN = 1;
        }

        // If we've received a UART character
        if (PIR1bits.RCIF)
        {
            c = getch();
            if (c == '\n')
            {
                putch(c);

                // If the mailbox door has been opened
                if (strncmp(inputBuf, "door", 4) == 0)
                {
                    // Clear the flag that indicates the door has been left open
                    // accidentally
                    doorOpen = false;
                    // If the transmitter has a low battery, it sent us a message
                    // before it sent us this 'door' message. So we don't want to
                    // clear the Low Battery flag unless the low battery message
                    // is more than 5 seconds old.
                    if (currentTime - lowBattTime > 5)
                        lowBatt = false;
                    flagUp = false;
                    mailInBox = false;
                    highLight = lowLight = -1;
                }

                else if (strncmp(inputBuf, "flup", 4) == 0)
                {
                    flagUp = true;
                }

                else if (strncmp(inputBuf, "fldn", 4) == 0)
                {
                    flagUp = false;
                }

                // Else if the battery is low
                else if (strncmp(inputBuf, "batt", 4) == 0)
                {
                    lowBatt = true;
                    lowBattTime = currentTime;
                }

                // Else if the mailbox door has been left open
                else if (strncmp(inputBuf, "open", 4) == 0)
                {
                    doorOpen = true;
                }

                // Else if we got a light sensor reading with the LED oN
                else if (inputBuf[0] == 'n')
                {
                    int lightReading = strtol(&inputBuf[1], &endPtr, 16);
                    if (endPtr == inputBuf + 4)
                    {
                        if (highLight == -1)
                            highLight = lightReading;
                        else
                            highLight = (lightReading + highLight) / 2;
                    }
                }

                // Else if we got a light sensor reading with the LED oFF
                // (lightReading is a high number)
                else if (inputBuf[0] == 'f')
                {
                    int lightReading = strtol(&inputBuf[1], &endPtr, 16);
                    if (endPtr == inputBuf + 4)
                    {
                        if (lowLight == -1)
                            lowLight = lightReading;
                        else
                            lowLight = (lightReading + lowLight) / 2;
                        if (highLight != -1)
                        {
                            if (highLight + 16 >= lowLight)
                                mailInBox = true;
                            else
                                mailInBox = false;
                        }
                    }
                }

                // Reset to the beginning of the input buffer
                pos = 0;
            }
            else if (c && isalnum(c))
            {
                putch(c);
                if (pos == sizeof(inputBuf))
                {
                    memmove(inputBuf, inputBuf + 1, sizeof(inputBuf) - 1);
                    --pos;
                }
                inputBuf[pos++] = c;
            }
        }

        // Figure out what to do with the flag
        if (doorOpen || lowBatt)
            newFlagState = WAVING;
        else if (mailInBox && !flagUp)
            newFlagState = UP;
        else
            newFlagState = DOWN;

        if (flagState != newFlagState)
        {
            flagState = newFlagState;
            if (flagState == DOWN)
                LowerFlag(currentTime);
            else if (flagState == UP)
                RaiseFlag(currentTime);
        }

        // If we're waving the flag (moving it up and down every 2 seconds)
        if (flagState == WAVING)
        {
            // If a second has passed since the last flag wave
            if (lastFlagWaveTime != currentTime)
            {
                lastFlagWaveTime = currentTime;
                // Every other second
                if (currentTime & 1)
                {
                    if (currentTime & 2)
                        RaiseFlag(currentTime);
                    else
                        LowerFlag(currentTime);
                }
            }
        }

        if (currentTime - flagMovedTime > 2)
            SERVO_POWER_SetLow();
    }
}

void LowerFlag(int currentTime)
{
    SERVO_POWER_SetHigh();
    EPWM2_LoadDutyValue(60);
    flagMovedTime = currentTime;
}

void RaiseFlag(int currentTime)
{
    SERVO_POWER_SetHigh();
    EPWM2_LoadDutyValue(34);
    flagMovedTime = currentTime;
}
