/*******************************************************************************
  Generated Main File 

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using MPLAB Code Configurator

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI. 
    Generation Information : 
        Product Revision  :  MPLAB Code Configurator - v1.0
        Device            :  PIC16F1825
        Version           :  1.1
    The generated drivers are tested against the following:
        Compiler          :  XC8 v1.21
        MPLAB             :  MPLAB X 1.90
*******************************************************************************/

/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/


#include "MCC_Generated_Files/mcc.h"
#include <string.h>

void SendMessage(const char *msg);

/****************************************************************************
 *
 *                        Main application
 *
****************************************************************************/
void main(void)
{
    int i = 0;
    adc_result_t lightSensorReading, batteryReading;
    char lightReading[5];
    char hexDigits[] = "0123456789ABCDEF";

    // initialize the device
    SYSTEM_Initialize();

    /**
     *                          CORE APPLICATION
     */

    // When the mailbox door opened, our power came on. First
    // thing to do is turn on another path for the power so we
    // can stay on after the door is closed.
    SUSTAIN_POWER_SetHigh();

    // Wait for power to stabilize
    __delay_ms(100);

    // Check the battery voltage. The Fixed Voltage Reference is 2.048 V. We read it
    // in the A/D converter against the reference voltage, which is the battery. If
    // FVR is more than 62% of the battery, then the battery is below 3.3 V.
    ADC_StartConversion(channel_FVR);
    while (!ADC_IsConversionDone())
    {
    }
    // Check the sensed value
    batteryReading = ADC_GetConversionResult();
    // Don't need the Fixed Voltage Reference anymore
    if (batteryReading > 0x3ff * 31 / 50)
        SendMessage("batt");
    FVRCON = 0;

    // Send a message that the door has been opened
    SendMessage("door");

    // Wait for two minutes while the mailman takes the outgoing mail, finds
    // the incoming mail, puts it in the box, adjusts the flag, and closes
    // the door
    for (i = 0; i < 120; ++i)
        __delay_ms(1000);

    // If the flag is up, then any mail inside is outgoing mail
    if (FLAG_UP_GetValue() == 1)
    {
        // Send a "flag down" message
        SendMessage("fldn");
    }
    else
    {
        // Send a "flag up" message
        SendMessage("flup");
    }

    // Check for the presence of mail. Turn on the IR LED, read the phototransistor
    // with an A/D conversion, turn the LED off and see if it's different. Do it
    // twice
    for (i = 0; i < 2; ++i)
    {
        // Turn on the IR LED for sensing the presence of mail.
        LIGHT_EMITTER_SetHigh();
        // Wait a bit for any latency in the LED or phototransistor (and
        // any capacitance in the circuit)
        __delay_ms(1);
        // If the phototransistor sees light, it'll pull the input
        // pin to ground. But if it doesn't see light, the input
        // stays high and there must be mail obstructing the light.
        // Alas, the IR LED isn't strong enough to saturate the phototransistor,
        // so we need to use the A/D converter instead of reading a logic 1 or 0
        ADC_StartConversion(LIGHT_SENSOR);
        while (!ADC_IsConversionDone())
        {
        }
        // Turn off the LED
        LIGHT_EMITTER_SetLow();
        // Check the sensed value
        lightSensorReading = ADC_GetConversionResult();
        lightReading[0] = 'n'; // 'n' for LED oN
        lightReading[1] = hexDigits[(lightSensorReading >> 8) & 0xf];
        lightReading[2] = hexDigits[(lightSensorReading >> 4) & 0xf];
        lightReading[3] = hexDigits[lightSensorReading & 0xf];
        SendMessage(lightReading);

        // Start a conversion with the LED off
        ADC_StartConversion(LIGHT_SENSOR);
        while (!ADC_IsConversionDone())
        {
        }
        // Check the sensed value
        lightSensorReading = ADC_GetConversionResult();
        lightReading[0] = 'f'; // 'f' for LED oFF
        lightReading[1] = hexDigits[(lightSensorReading >> 8) & 0xf];
        lightReading[2] = hexDigits[(lightSensorReading >> 4) & 0xf];
        lightReading[3] = hexDigits[lightSensorReading & 0xf];
        SendMessage(lightReading);
    }


    // Allow us to turn off when the mailbox door closes
    SUSTAIN_POWER_SetLow();

    // Wait for 5 seconds (to make sure any capacitance drains out from
    // our power supply)
    for (i = 0; i < 5; ++i)
        __delay_ms(1000);

    // If we're still running, the door must've been left open.
    // Send a message
    SendMessage("open");

    // Save power: turn off the UART. This should turn off the transmitter too
    TXSTA = 0;
    RCSTA = 0;

    // As long as we have power from the door being open, sleep
    while (true)
    {
        SLEEP();
    }
}

void SendMessage(const char *msg)
{
    for (int i = 0; i < 15; ++i)
        puts(msg);
}
